import os
import glob
from openbabel import openbabel
from colorama import init, Fore, Style

# Initialize colorama for cross-platform support
init(autoreset=True)

def convert_sdf_to_pdbqt():
    # Automatically detect the 'sdf_ligands' folder in the current directory
    current_dir = os.getcwd()
    sdf_folder = os.path.join(current_dir, "sdf_ligands")
    
    # Check if the 'sdf_ligands' folder exists
    if not os.path.exists(sdf_folder):
        print(f"{Fore.RED}Error: 'sdf_ligands' folder not found in the current directory.{Style.RESET_ALL}")
        return

    # Create the output folder in the current directory
    output_folder = os.path.join(current_dir, "ligand_pdbqt")
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Loop over all SDF files in the 'sdf_ligands' folder
    sdf_files = glob.glob(os.path.join(sdf_folder, "*.sdf"))
    
    if not sdf_files:
        print(f"{Fore.RED}No SDF files found in 'sdf_ligands' folder.{Style.RESET_ALL}")
        return
    
    successful_conversions = 0
    ob_conversion = openbabel.OBConversion()
    ob_conversion.SetInAndOutFormats("sdf", "pdbqt")
    
    for sdf_file in sdf_files:
        base_name = os.path.basename(sdf_file)  # Get only the filename
        output_file = os.path.join(output_folder, base_name.replace(".sdf", ".pdbqt"))
        
        mol = openbabel.OBMol()

        # Read the input SDF file
        if not ob_conversion.ReadFile(mol, sdf_file):
            print(f"{Fore.RED}Failed to read {base_name}{Style.RESET_ALL}")
            continue

        # Add hydrogens and calculate Gasteiger charges
        mol.AddHydrogens()
        openbabel.OBChargeModel.FindType("gasteiger").ComputeCharges(mol)
        
        # Write to PDBQT format
        if ob_conversion.WriteFile(mol, output_file):
            output_filename = os.path.basename(output_file)  # Only display the filename
            print(f"{Fore.BLUE}Converted {base_name} to {output_filename}{Style.RESET_ALL}")
            successful_conversions += 1
        else:
            print(f"{Fore.RED}Failed to convert {base_name}{Style.RESET_ALL}")
    
    # Print summary of successful conversions
    print(f"{Fore.GREEN}{successful_conversions} molecules converted successfully!{Style.RESET_ALL}")

if __name__ == "__main__":
    convert_sdf_to_pdbqt()
