import os
import pubchempy as pcp
import wikipedia
from rdkit import Chem
from rdkit.Chem import MolToSmiles, MolFromSmiles

# Function to search for the compound on PubChem using PubChemPy
def search_pubchem(compound_name):
    try:
        compounds = pcp.get_compounds(compound_name, 'name')
        if compounds:
            compound = compounds[0]  # Use the best match
            compound_id = compound.cid
            iupac_name = compound.iupac_name
            compound_url = f"https://pubchem.ncbi.nlm.nih.gov/compound/{compound_id}"
            return compound_id, iupac_name, compound_url
    except Exception as e:
        print(f"Error fetching data from PubChem: {e}")
    return None, None, None

# Function to search Wikipedia for PubChem CID
def get_pubchem_cid_from_wikipedia(compound_name):
    try:
        print(f"Attempting to find {compound_name} via Wikipedia search...")
        search_results = wikipedia.search(f"{compound_name} PubChem CID", results=5)
        
        for result in search_results:
            try:
                page = wikipedia.page(result)
                if 'PubChem CID' in page.content:
                    # Extract PubChem CID from the content
                    soup = BeautifulSoup(page.content, 'html.parser')
                    for ref in soup.find_all('a', href=True):
                        href = ref['href']
                        if 'pubchem.ncbi.nlm.nih.gov/compound/' in href:
                            cid = href.split('/')[-1]
                            print(f"Found PubChem CID: {cid}")
                            return cid
            except wikipedia.exceptions.DisambiguationError as e:
                print(f"Disambiguation error: {e.options}")
            except wikipedia.exceptions.PageError as e:
                print(f"Page error: {e}")
    except wikipedia.exceptions.HTTPTimeoutError as e:
        print(f"HTTP timeout error: {e}")
    except Exception as e:
        print(f"Error during Wikipedia search: {e}")
    return None

# Function to convert name to SMILES using RDKit
def get_smiles_from_rdkit(compound_name):
    try:
        # RDKit doesn't directly convert names to SMILES, but you can create a simple map if you have a list
        # Here, assume you have a way to get a molecule from the name, e.g., a local database or predefined map
        # For demonstration, let's use a placeholder SMILES dictionary
        smiles_dict = {
            "DMT": "CN1C=CC2=C1C(=O)N(C2=O)C",
            "cocaine": "CN1C2=C(C=C1)C(=O)N(C2=O)C",
            # Add more mappings here if necessary
        }
        return smiles_dict.get(compound_name, None)
    except Exception as e:
        print(f"Error getting SMILES from RDKit: {e}")
    return None

# Function to download SDF from PubChem
def download_sdf_pubchem(compound_id, compound_name):
    try:
        # Define the file path where SDF will be saved
        file_path = os.path.join("sdf_ligands", f"{compound_name.replace(' ', '_')}.sdf")
        
        # Download the SDF file from PubChem
        pcp.download('SDF', file_path, identifier=compound_id, overwrite=True)
        return True
    except Exception as e:
        print(f"Error downloading SDF file from PubChem: {e}")
    return False

# Function to fetch information if PubChem and Wikipedia fail
def fetch_info_with_rdkit(compound_name):
    smiles = get_smiles_from_rdkit(compound_name)
    if smiles:
        try:
            mol = Chem.MolFromSmiles(smiles)
            if mol:
                inchi_str = Chem.MolToInchi(mol)
                return inchi_str, smiles
        except Exception as e:
            print(f"Error converting SMILES with RDKit: {e}")
    return None, None

# Main function
def main():
    # Read the screening list
    with open("screening_list.txt", "r") as file:
        compounds = [line.strip() for line in file.readlines() if line.strip()]

    # Create directory for SDF files
    if not os.path.exists("sdf_ligands"):
        os.makedirs("sdf_ligands")

    log_entries = []

    for compound in compounds:
        print(f"Processing compound: {compound}")
        compound_id, iupac_name, pubchem_url = search_pubchem(compound)

        if not compound_id:
            # If not found, attempt to find PubChem CID via Wikipedia search
            pubchem_cid = get_pubchem_cid_from_wikipedia(compound)
            if pubchem_cid:
                compound_id, iupac_name, pubchem_url = search_pubchem(pubchem_cid)
                if not compound_id:
                    # If still not found, use RDKit as a fallback
                    inchi_str, smiles = fetch_info_with_rdkit(compound)
                    if not inchi_str:
                        print(f"Could not find {compound} on PubChem or via Wikipedia search.")
                        log_entries.append(f"List Name: {compound}\nIUPAC: Not found\nLink: Not found")
                        continue
                    else:
                        pubchem_url = "Not available"
            else:
                inchi_str, smiles = fetch_info_with_rdkit(compound)
                if not inchi_str:
                    print(f"Could not find {compound} on PubChem or via Wikipedia search.")
                    log_entries.append(f"List Name: {compound}\nIUPAC: Not found\nLink: Not found")
                    continue
                pubchem_url = "Not available"

        if compound_id:
            success = download_sdf_pubchem(compound_id, compound)
            if success:
                log_entries.append(f"List Name: {compound}\nIUPAC: {iupac_name}\nLink: {pubchem_url}")
            else:
                log_entries.append(f"List Name: {compound}\nIUPAC: {iupac_name}\nLink: Error downloading SDF")
        else:
            log_entries.append(f"List Name: {compound}\nIUPAC: {iupac_name}\nLink: {pubchem_url}")

    # Write log file
    with open("sdf_ligands/log.txt", "w") as log_file:
        log_file.write("\n\n".join(log_entries))
    print("Process completed. Check 'sdf_ligands' directory for results.")

if __name__ == "__main__":
    main()
