import os

def extract_and_save_conformations(input_folder, threshold=None, top_n=None, top_n_per_ligand=False):
    """
    Extract conformations from .dlg files and save them as .pdbqt files.
    """
    # Define the output folder based on the extraction method
    if threshold is not None:
        output_folder = f'filtered_results_BE{threshold:.1f}'
    elif top_n is not None:
        if top_n_per_ligand:
            output_folder = f'top_{top_n}_results_per_ligand'
        else:
            output_folder = f'top_{top_n}_results'
    else:
        output_folder = 'results'

    # Ensure the output folder exists
    os.makedirs(output_folder, exist_ok=True)
    
    # Get all .dlg files in the input folder
    dlg_files = [f for f in os.listdir(input_folder) if f.endswith('.dlg')]
    
    # Check if any .dlg files were found
    if not dlg_files:
        print("No .dlg files found in the input folder.")
        return
    
    def extract_conformations(file_path):
        conformations = []
        with open(file_path, 'r') as file:
            lines = file.readlines()
            current_conformation = []
            current_energy = None
            in_conformation = False
            
            for line in lines:
                if line.startswith("DOCKED: MODEL"):
                    in_conformation = True
                    current_conformation = [line]
                elif line.startswith("DOCKED: ENDMDL") and in_conformation:
                    current_conformation.append(line)
                    if current_energy is not None:
                        conformations.append((current_energy, current_conformation))
                    current_energy = None
                    in_conformation = False
                elif in_conformation:
                    current_conformation.append(line)
                    if "Estimated Free Energy of Binding" in line:
                        parts = line.split('=')
                        try:
                            current_energy = float(parts[1].split()[0].strip())
                        except ValueError:
                            current_energy = None
        return conformations
    
    all_conformations = []
    
    # Process each .dlg file
    for dlg_file in dlg_files:
        ligand_name = dlg_file.replace('.dlg', '')
        file_path = os.path.join(input_folder, dlg_file)
        conformations = extract_conformations(file_path)
        
        if threshold is not None:
            # Save conformations that meet the threshold
            conformations = [(energy, conf) for energy, conf in conformations if energy <= threshold]
        
        if top_n_per_ligand:
            # Sort and select top N conformations for this ligand
            conformations = sorted(conformations, key=lambda x: x[0])[:top_n]
        
        all_conformations.append((ligand_name, conformations))
    
    if top_n is not None and not top_n_per_ligand:
        # Flatten all conformations and select the top N across all ligands
        all_conformations = [(ligand, energy, conf) for ligand, conformation_list in all_conformations for energy, conf in conformation_list]
        all_conformations = sorted(all_conformations, key=lambda x: x[1])[:top_n]
    
    # Save conformations
    energy_counts = {}
    
    if top_n_per_ligand or threshold is not None:
        for ligand_name, conformations in all_conformations:
            for energy, conformation in conformations:
                energy_str = "{:.1f}".format(energy)
                if energy_str not in energy_counts:
                    energy_counts[energy_str] = 0
                energy_counts[energy_str] += 1
                
                conformation_filename = '{}_BE{}_conf{}.pdbqt'.format(
                    ligand_name, energy_str, energy_counts[energy_str])
                conformation_path = os.path.join(output_folder, conformation_filename)
                
                with open(conformation_path, 'w') as out_file:
                    out_file.writelines(conformation)
    else:
        for ligand_name, energy, conformation in all_conformations:
            energy_str = "{:.1f}".format(energy)
            if energy_str not in energy_counts:
                energy_counts[energy_str] = 0
            energy_counts[energy_str] += 1
            
            conformation_filename = '{}_BE{}_conf{}.pdbqt'.format(
                ligand_name, energy_str, energy_counts[energy_str])
            conformation_path = os.path.join(output_folder, conformation_filename)
            
            with open(conformation_path, 'w') as out_file:
                out_file.writelines(conformation)
    
    print(f"Conformations have been saved in the '{output_folder}' folder.")
    return output_folder  # Return the output folder for use in subsequent steps

def remove_docked_prefix(pdbqt_folder):
    """
    Remove the 'DOCKED:' prefix from all lines in .pdbqt files in the specified folder.
    """
    if not os.path.exists(pdbqt_folder):
        print(f"Folder '{pdbqt_folder}' does not exist.")
        return
    
    for filename in os.listdir(pdbqt_folder):
        if filename.endswith('.pdbqt'):
            file_path = os.path.join(pdbqt_folder, filename)
            with open(file_path, 'r') as file:
                lines = file.readlines()

            # Remove only the 'DOCKED:' prefix and keep the rest of the line intact
            cleaned_lines = [line.replace("DOCKED: ", "", 1) if line.startswith("DOCKED: ") else line for line in lines]
            
            with open(file_path, 'w') as file:
                file.writelines(cleaned_lines)
    
    print(f"Prefix 'DOCKED:' removed from all .pdbqt files in the '{pdbqt_folder}' folder.")

def clean_user_lines(pdbqt_folder):
    """
    Remove lines that are entirely 'USER' entries from .pdbqt files, while keeping meaningful lines intact.
    """
    if not os.path.exists(pdbqt_folder):
        print(f"Folder '{pdbqt_folder}' does not exist.")
        return

    for filename in os.listdir(pdbqt_folder):
        if filename.endswith('.pdbqt'):
            file_path = os.path.join(pdbqt_folder, filename)
            with open(file_path, 'r') as file:
                lines = file.readlines()

            # Keep lines that don't start with 'USER' or 'DOCKED: USER'
            cleaned_lines = [line for line in lines if not (line.strip().startswith("USER") or line.strip().startswith("DOCKED: USER"))]
            
            with open(file_path, 'w') as file:
                file.writelines(cleaned_lines)

    print(f"Removed 'USER' lines from all .pdbqt files in the '{pdbqt_folder}' folder.")

def main():
    input_folder = os.getcwd()
    
    # Prompt the user to choose between threshold or top N results
    print("Choose the extraction method:")
    print("1. Extract conformations based on a binding affinity threshold.")
    print("2. Extract the top N conformations.")
    
    choice = input("Enter 1 or 2: ")
    
    if choice == "1":
        try:
            threshold = float(input("Enter the binding affinity threshold in kcal/mol (e.g. -8): "))
        except ValueError:
            print("Invalid input. Please enter a numeric value for the threshold.")
            exit(1)
        output_folder = extract_and_save_conformations(input_folder, threshold=threshold)
    
    elif choice == "2":
        try:
            top_n = int(input("Enter the number of top conformations to extract: "))
        except ValueError:
            print("Invalid input. Please enter an integer value for the number of conformations.")
            exit(1)
        
        print("Do you want the top conformations from each ligand file or across all ligands?")
        print("1. Top conformations across all ligands.")
        print("2. Top conformations from each ligand file.")
        
        sub_choice = input("Enter 1 or 2: ")
        
        if sub_choice == "1":
            output_folder = extract_and_save_conformations(input_folder, top_n=top_n)
        elif sub_choice == "2":
            output_folder = extract_and_save_conformations(input_folder, top_n=top_n, top_n_per_ligand=True)
        else:
            print("Invalid choice.")
            exit(1)
    
    # Remove 'DOCKED:' prefix from the saved .pdbqt files
    remove_docked_prefix(output_folder)
    
    # Remove 'USER' lines from the saved .pdbqt files
    clean_user_lines(output_folder)

if __name__ == '__main__':
    main()
