# LightningAutoDock (LAD) Tools: Streamlined Ligand Screening with AutoDock4

**BKEA iGEM 2024**

Welcome to the **LightningAutoDock (LAD) Tools** repository! LAD Tools is a Python-based software suite designed to automate and simplify the molecular docking process using AutoDock4. This suite enables high-throughput ligand screening by automating key steps in ligand acquisition, preparation, docking, and result analysis, saving researchers significant time and reducing the errors commonly encountered with existing tools.

## Description

LAD Tools was born out of the need to overcome the steep learning curve and inefficiencies of current molecular docking software like AutoDock4 and AutoDockTools (ADT). Our goal was to create a powerful, accessible, and error-tolerant pipeline to support ligand screening using AutoDock4, with an emphasis on scalability, high-throughput screening, and user-friendliness.

The LAD Tools suite consists of four main components:

1. **LightningScreen**: Automatically sources ligand structure files from public chemical databases.
2. **LightningBabel**: Prepares ligands for docking by converting files to PDBQT format.
3. **LightningAutoDock**: Automates the docking process with AutoDock4, including parallelization for high-throughput screening.
4. **LightningViewer**: Filters and sorts docking results based on binding affinities for post-docking analysis.

These tools are designed to handle large numbers of ligands and docking simulations, making it easier for researchers to perform large-scale experiments.

## Installation

### Prerequisites

Before you install the LAD Tools, you must first install the following software from their official websites. These are core components required to run molecular docking simulations using AutoDock4:

1. **MGLTools** (v1.5.7): This includes AutoDockTools (ADT), which is necessary for preparing docking simulations.
   - Download: [MGLTools Official Page](http://mgltools.scripps.edu/downloads)
   - Ensure you install **Python 2.7.11** which comes bundled with MGLTools.

2. **AutoDock4**: The docking engine used for simulations.
   - Download: [AutoDock4 Official Page](http://autodock.scripps.edu/downloads/autodock-registration/autodock-4-2-download-page)
   - Install the program and ensure they are located in the workspace. See the folder structure below.

3. **OpenBabel**: Used for ligand file conversions (e.g., SDF to PDBQT).
   - Download: [OpenBabel Official Page](https://openbabel.org/wiki/Main_Page)
   - Install the program and ensure it is in your system’s PATH.

4. **Python 3**: Many of the lightning tools require the newest version of python. Python 2.7.11 won't work here.
   - Download: Python 3 from pythons official website

### Python Libraries

In addition to the above tools, LAD Tools relies on several Python libraries. These can be installed via pip:
   - pip install pubchempy wikipedia-api rdkit colorama
   - pip install openbabel-wheel

Make sure that AutoDock4, MGLTools, and OpenBabel are in your system’s PATH and that Python 2.7 (bundled with MGLTools) is accessible in the environment for running `lightningautodock2.py`.

## Folder Structure

LAD Tools expects a specific folder structure to function properly:
```
/workspace
│
├── /sdf_ligands       # Place your SDF ligand files here (created by LightningScreen)
├── /ligand_pdbqt      # Output directory for PDBQT files (created by LightningBabel)
├── /top_results       # Directory where ranked results are placed (created by LightningViewer)
├── /filtered_results   # Directory for filtered results (created by LightningViewer)
│
├── autodock4.exe      # AutoDock4 executable for docking simulations
└── autogrid4.exe      # AutoGrid4 executable for grid map generation
```

Beware that all ligand and protein files must be the workspace directory and not in subfolders. LightningAutoDock.py needs access to the files from the working directory. Also the protein needs to be named to "receptor.pdbqt".

## Usage

### LightningScreen: Source Ligand Structures

LightningScreen automates the retrieval of ligand structures from public databases like PubChem.

1. Prepare a `screening_list.txt` file with a list of compound names or PubChem CIDs.
2. Run the script: ``` python3 lightningScreen.py ```
3. The downloaded SDF files will be saved in the `/sdf_ligands` folder.

### LightningBabel: Prepare Ligands for Docking

After sourcing ligands, convert the SDF files to PDBQT format.

1. Place the SDF files in the `/sdf_ligands` directory.
2. Run the script: ``` python3 lightningbabel.py ```
3. The PDBQT files will be saved in `/ligand_pdbqt`.

### LightningAutoDock: Perform Docking

This is the core tool for running parallelized docking simulations.

1. Place your receptor PDBQT file in the workspace.
2. Run the script: ``` python2 lightningautodock2.py ```
3. The docking results will be saved in the `/docking_results` folder.

### LightningViewer: Filter Docking Results

Filter the docking results based on binding affinity thresholds.

1. Place your DLG output files in the workspace.
2. Run the script: ``` python3 lightningviewer.py ```
3. The filtered results will be saved in the `/filtered_results` folder.

## Contributing

We welcome contributions from the scientific community! If you have suggestions, bug reports, or would like to contribute improvements, please open an issue or submit a pull request on our GitLab page.

### Steps to Contribute:

1. Fork the repository.
2. Create a new branch for your feature (`git checkout -b feature-name`).
3. Commit your changes (`git commit -m 'Add new feature'`).
4. Push the branch to your fork (`git push origin feature-name`).
5. Open a pull request.

## Authors and Acknowledgments

This project was developed by the drylab teamleader of BKEA iGEM 2024.

## Software Credits and Citations

- **MGLTools**: A suite developed at the Scripps Research Institute for molecular visualization and docking preparation.  
  Citation: Sanner, M. F., et al. "MGLTools: a molecular graphics tool for the visualization and analysis of molecular structures." Center for Computational Structural Biology, Scripps Research Institute.

- **AutoDock4**: Widely used for predicting ligand-receptor interactions.  
  Citation: Morris, G. M., et al. "AutoDock4: An improved version of AutoDock that expands the range of ligand conformations." *Journal of Computational Chemistry*, 30(16): 2785–2791, 2009.

- **AutoGrid4**: Used with AutoDock to generate grid maps for docking.  
  Citation: Goodsell, D. S., & Olson, A. J. "Automated docking of substrates to proteins by simulated annealing." *Journal of Molecular Graphics*, 14(3): 199–203, 1996.

- **OpenBabel**: An open-source chemical toolbox for file conversion and cheminformatics.  
  Citation: O'Boyle, N. M., et al. "Open Babel: An open chemical toolbox." *Journal of Cheminformatics*, 3:33, 2011.